@@ -34,9 +34,9 @@ impl Value {
             VarType::Auto => parse_i32(&s),
             VarType::Boolean => {
                 let raw = s.to_uppercase();
-                if raw == "TRUE" {
+                if raw == "TRUE" || raw == "YES" || raw == "Y" {
                     Ok(Value::Boolean(true))
-                } else if raw == "FALSE" {
+                } else if raw == "FALSE" || raw == "NO" || raw == "N" {
                     Ok(Value::Boolean(false))
                 } else {
                     bail!("Invalid boolean literal {}", s);
@@ -193,6 +193,15 @@ impl Value {
         }
     }
 
+    /// Performs a modulo operation.
+    pub fn modulo(&self, other: &Self) -> Fallible<Self> {
+        match (self, other) {
+            (Value::Integer(_), Value::Integer(0)) => bail!("Modulo by zero"),
+            (Value::Integer(lhs), Value::Integer(rhs)) => Ok(Value::Integer(lhs % rhs)),
+            (_, _) => Err(format_err!("Cannot modulo {:?} by {:?}", self, other)),
+        }
+    }
+
     /// Performs an integer negation.
     pub fn neg(&self) -> Fallible<Self> {
         match self {
@@ -242,6 +251,7 @@ impl Expr {
             Expr::Subtract(lhs, rhs) => Value::sub(&lhs.eval(vars)?, &rhs.eval(vars)?),
             Expr::Multiply(lhs, rhs) => Value::mul(&lhs.eval(vars)?, &rhs.eval(vars)?),
             Expr::Divide(lhs, rhs) => Value::div(&lhs.eval(vars)?, &rhs.eval(vars)?),
+            Expr::Modulo(lhs, rhs) => Value::modulo(&lhs.eval(vars)?, &rhs.eval(vars)?),
             Expr::Negate(e) => Value::neg(&e.eval(vars)?),
         }
     }
@@ -271,36 +281,26 @@ mod tests {
     fn test_value_parse_as_boolean() {
         use super::Value::*;
 
-        assert_eq!(
-            Boolean(true),
-            Value::parse_as(VarType::Boolean, "true").unwrap()
-        );
-        assert_eq!(
-            Boolean(true),
-            Value::parse_as(VarType::Boolean, "TrUe").unwrap()
-        );
-        assert_eq!(
-            Boolean(true),
-            Value::parse_as(VarType::Boolean, "TRUE").unwrap()
-        );
+        for s in &["true", "TrUe", "TRUE", "yes", "Yes", "y", "Y"] {
+            assert_eq!(
+                Boolean(true),
+                Value::parse_as(VarType::Boolean, *s).unwrap()
+            );
+        }
 
-        assert_eq!(
-            Boolean(false),
-            Value::parse_as(VarType::Boolean, "false").unwrap()
-        );
-        assert_eq!(
-            Boolean(false),
-            Value::parse_as(VarType::Boolean, "FaLsE").unwrap()
-        );
-        assert_eq!(
-            Boolean(false),
-            Value::parse_as(VarType::Boolean, "FALSE").unwrap()
-        );
+        for s in &["false", "FaLsE", "FALSE", "no", "No", "n", "N"] {
+            assert_eq!(
+                Boolean(false),
+                Value::parse_as(VarType::Boolean, *s).unwrap()
+            );
+        }
 
-        assert_eq!(
-            "Invalid boolean literal yes",
-            format!("{}", Value::parse_as(VarType::Boolean, "yes").unwrap_err())
-        );
+        for s in &["ye", "0", "1", " true"] {
+            assert_eq!(
+                format!("Invalid boolean literal {}", s),
+                format!("{}", Value::parse_as(VarType::Boolean, *s).unwrap_err())
+            );
+        }
     }
 
     #[test]
@@ -742,6 +742,37 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_value_modulo() {
+        use super::Value::*;
+
+        assert_eq!(
+            "Cannot modulo Boolean(false) by Boolean(true)",
+            format!("{}", Boolean(false).modulo(&Boolean(true)).unwrap_err())
+        );
+
+        assert_eq!(Integer(0), Integer(10).modulo(&Integer(5)).unwrap());
+        assert_eq!(Integer(2), Integer(20).modulo(&Integer(3)).unwrap());
+        assert_eq!(
+            "Modulo by zero",
+            format!("{}", Integer(4).modulo(&Integer(0)).unwrap_err())
+        );
+        assert_eq!(
+            "Cannot modulo Integer(4) by Text(\"5\")",
+            format!("{}", Integer(4).modulo(&Text("5".to_owned())).unwrap_err())
+        );
+
+        assert_eq!(
+            "Cannot modulo Text(\"\") by Text(\"a\")",
+            format!(
+                "{}",
+                Text("".to_owned())
+                    .modulo(&Text("a".to_owned()))
+                    .unwrap_err()
+            )
+        );
+    }
+
     #[test]
     fn test_value_neg() {
         use super::Value::*;
