@@ -46,6 +46,7 @@ pub enum Token {
     Minus,
     Multiply,
     Divide,
+    Modulo,
 
     Equal,
     NotEqual,
@@ -77,12 +78,17 @@ trait CharOps {
     /// Use this instead of `is_whitespace`, which accounts for newlines but we need to handle
     /// those explicitly.
     fn is_space(&self) -> bool;
+
+    /// Returns true if the character can be part of an identifier.
+    fn is_word(&self) -> bool;
 }
 
 impl CharOps for char {
     fn is_separator(&self) -> bool {
         match *self {
-            '\n' | '(' | ')' | '\'' | '=' | '<' | '>' | ';' | ',' | '+' | '-' | '*' | '/' => true,
+            '\n' | ':' | '(' | ')' | '\'' | '=' | '<' | '>' | ';' | ',' | '+' | '-' | '*' | '/' => {
+                true
+            }
             ch => ch.is_space(),
         }
     }
@@ -95,6 +101,13 @@ impl CharOps for char {
             _ => false,
         }
     }
+
+    fn is_word(&self) -> bool {
+        match *self {
+            '_' => true,
+            ch => ch.is_alphanumeric(),
+        }
+    }
 }
 
 /// Iterator over the tokens of the language.
@@ -195,7 +208,7 @@ impl<'a> Lexer<'a> {
         let mut vtype = VarType::Auto;
         loop {
             match self.input.peek() {
-                Some(Ok(ch)) if ch.is_alphanumeric() => s.push(self.input.next().unwrap()?),
+                Some(Ok(ch)) if ch.is_word() => s.push(self.input.next().unwrap()?),
                 Some(Ok(ch)) if ch.is_separator() => break,
                 Some(Ok('?')) => {
                     vtype = VarType::Boolean;
@@ -227,6 +240,7 @@ impl<'a> Lexer<'a> {
             "END" => Ok(Token::End),
             "FALSE" => Ok(Token::Boolean(false)),
             "IF" => Ok(Token::If),
+            "MOD" => Ok(Token::Modulo),
             "NOT" => Ok(Token::Not),
             "OR" => Ok(Token::Or),
             "REM" => self.consume_rest_of_line(),
@@ -306,7 +320,7 @@ impl<'a> Lexer<'a> {
         }
         let ch = ch.unwrap();
         match ch {
-            '\n' => Ok(Token::Eol),
+            '\n' | ':' => Ok(Token::Eol),
             '\'' => self.consume_rest_of_line(),
 
             '"' => self.consume_text('"'),
@@ -326,7 +340,7 @@ impl<'a> Lexer<'a> {
             '<' | '>' => self.consume_operator(ch),
 
             ch if ch.is_digit(10) => self.consume_integer(ch),
-            ch if ch.is_alphabetic() => self.consume_symbol(ch),
+            ch if ch.is_word() => self.consume_symbol(ch),
             ch => self.handle_bad_read(format!("Unknown character: {}", ch)),
         }
     }
@@ -396,8 +410,8 @@ mod tests {
     ///
     /// `Token::Eof` should not be provided in `exp_tokens` as this explicitly waits for that.
     fn do_ok_test(input: &str, exp_tokens: &[Token]) {
-        let mut cursor = io::Cursor::new(input.as_bytes());
-        let mut lexer = Lexer::from(&mut cursor);
+        let mut input = input.as_bytes();
+        let mut lexer = Lexer::from(&mut input);
 
         let mut tokens = vec![];
         loop {
@@ -413,8 +427,8 @@ mod tests {
 
     #[test]
     fn test_empty() {
-        let mut cursor = io::Cursor::new(b"");
-        let mut lexer = Lexer::from(&mut cursor);
+        let mut input = b"".as_ref();
+        let mut lexer = Lexer::from(&mut input);
         assert_eq!(Token::Eof, lexer.read().unwrap());
         assert_eq!(Token::Eof, lexer.read().unwrap());
     }
@@ -432,6 +446,7 @@ mod tests {
     #[test]
     fn test_multiple_lines() {
         do_ok_test("   \n \t   \n  ", &[Token::Eol, Token::Eol]);
+        do_ok_test("   : \t   :  ", &[Token::Eol, Token::Eol]);
     }
 
     /// Syntactic sugar to instantiate a `VarRef` without an explicity type annotation.
@@ -442,7 +457,7 @@ mod tests {
     #[test]
     fn test_some_tokens() {
         do_ok_test(
-            "123 45 \n 6 abc a38z\na=3",
+            "123 45 \n 6 abc a38z: a=3 with_underscores_1=_2",
             &[
                 Token::Integer(123),
                 Token::Integer(45),
@@ -454,6 +469,28 @@ mod tests {
                 new_auto_symbol("a"),
                 Token::Equal,
                 Token::Integer(3),
+                new_auto_symbol("with_underscores_1"),
+                Token::Equal,
+                new_auto_symbol("_2"),
+            ],
+        );
+    }
+
+    #[test]
+    fn test_boolean_literals() {
+        do_ok_test(
+            "true TRUE yes YES y false FALSE no NO n",
+            &[
+                Token::Boolean(true),
+                Token::Boolean(true),
+                new_auto_symbol("yes"),
+                new_auto_symbol("YES"),
+                new_auto_symbol("y"),
+                Token::Boolean(false),
+                Token::Boolean(false),
+                new_auto_symbol("no"),
+                new_auto_symbol("NO"),
+                new_auto_symbol("n"),
             ],
         );
     }
@@ -479,6 +516,11 @@ mod tests {
             "REM This is a comment\nNOT 'This is another comment\n",
             &[Token::Eol, Token::Not, Token::Eol],
         );
+
+        do_ok_test(
+            "REM This is a comment: and the colon doesn't yield Eol\nNOT 'Another: comment\n",
+            &[Token::Eol, Token::Not, Token::Eol],
+        );
     }
 
     #[test]
@@ -504,6 +546,13 @@ mod tests {
             ],
         );
 
+        do_ok_test(
+            " \"this is a string with ; special : characters in it\"",
+            &[Token::Text(
+                "this is a string with ; special : characters in it".to_owned(),
+            )],
+        );
+
         do_ok_test(
             "\"this \\\"is escaped\\\" \\\\ \\a\" 1",
             &[
@@ -571,6 +620,8 @@ mod tests {
         do_operator_test("-", Token::Minus);
         do_operator_test("*", Token::Multiply);
         do_operator_test("/", Token::Divide);
+        do_operator_test("MOD", Token::Modulo);
+        do_operator_test("mod", Token::Modulo);
     }
 
     #[test]
@@ -608,8 +659,8 @@ mod tests {
 
     #[test]
     fn test_peekable_lexer() {
-        let mut cursor = io::Cursor::new(b"a b 123");
-        let mut lexer = Lexer::from(&mut cursor).peekable();
+        let mut input = b"a b 123".as_ref();
+        let mut lexer = Lexer::from(&mut input).peekable();
         assert_eq!(&new_auto_symbol("a"), lexer.peek().unwrap());
         assert_eq!(&new_auto_symbol("a"), lexer.peek().unwrap());
         assert_eq!(new_auto_symbol("a"), lexer.read().unwrap());
@@ -643,10 +694,10 @@ mod tests {
         );
 
         do_ok_test(
-            "a b_d 5",
+            "a b|d 5",
             &[
                 new_auto_symbol("a"),
-                Token::Bad("Unexpected character in symbol: _".to_owned()),
+                Token::Bad("Unexpected character in symbol: |".to_owned()),
                 Token::Integer(5),
             ],
         );
