@@ -28,11 +28,13 @@
 #[macro_use]
 extern crate failure;
 
-mod ast;
-mod eval;
-mod exec;
+// TODO(jmmv): Should narrow the exposed interface by 1.0.0.
+pub mod ast;
+pub mod console;
+pub mod eval;
+pub mod exec;
 mod lexer;
 mod parser;
+mod program;
 mod reader;
-
-pub use exec::{Console, Machine};
+pub mod repl;
