@@ -36,6 +36,7 @@ enum ExprOp {
     Subtract,
     Multiply,
     Divide,
+    Modulo,
     Negate,
 
     Equal,
@@ -64,6 +65,7 @@ impl ExprOp {
             Token::Plus => ExprOp::Add,
             Token::Multiply => ExprOp::Multiply,
             Token::Divide => ExprOp::Divide,
+            Token::Modulo => ExprOp::Modulo,
             Token::And => ExprOp::And,
             Token::Or => ExprOp::Or,
             Token::Xor => ExprOp::Xor,
@@ -83,6 +85,7 @@ impl ExprOp {
 
             ExprOp::Multiply => 3,
             ExprOp::Divide => 3,
+            ExprOp::Modulo => 3,
 
             ExprOp::Add => 2,
             ExprOp::Subtract => 2,
@@ -122,6 +125,7 @@ impl ExprOp {
             ExprOp::Subtract => apply2(exprs, Expr::Subtract),
             ExprOp::Multiply => apply2(exprs, Expr::Multiply),
             ExprOp::Divide => apply2(exprs, Expr::Divide),
+            ExprOp::Modulo => apply2(exprs, Expr::Modulo),
             ExprOp::Equal => apply2(exprs, Expr::Equal),
             ExprOp::NotEqual => apply2(exprs, Expr::NotEqual),
             ExprOp::Less => apply2(exprs, Expr::Less),
@@ -298,6 +302,7 @@ impl<'a> Parser<'a> {
                 | Token::Plus
                 | Token::Multiply
                 | Token::Divide
+                | Token::Modulo
                 | Token::And
                 | Token::Or
                 | Token::Xor => {
@@ -521,8 +526,8 @@ mod tests {
     /// Runs the parser on the given `input` and expects the returned statements to match
     /// `exp_statements`.
     fn do_ok_test(input: &str, exp_statements: &[Statement]) {
-        let mut cursor = io::Cursor::new(input.as_bytes());
-        let mut parser = Parser::from(&mut cursor);
+        let mut input = input.as_bytes();
+        let mut parser = Parser::from(&mut input);
 
         let mut statements = vec![];
         loop {
@@ -538,8 +543,8 @@ mod tests {
 
     /// Runs the parser on the given `input` and expects the `err` error message.
     fn do_error_test(input: &str, expected_err: &str) {
-        let mut cursor = io::Cursor::new(input.as_bytes());
-        let mut parser = Parser::from(&mut cursor);
+        let mut input = input.as_bytes();
+        let mut parser = Parser::from(&mut input);
         assert_eq!(
             expected_err,
             format!("{}", parser.parse().expect_err("Parsing did not fail"))
@@ -553,8 +558,8 @@ mod tests {
     // TODO(jmmv): Need better testing to ensure the parser is reset to something that can be
     // parsed next.
     fn do_error_test_no_reset(input: &str, expected_err: &str) {
-        let mut cursor = io::Cursor::new(input.as_bytes());
-        let mut parser = Parser::from(&mut cursor);
+        let mut input = input.as_bytes();
+        let mut parser = Parser::from(&mut input);
         assert_eq!(
             expected_err,
             format!("{}", parser.parse().expect_err("Parsing did not fail"))
@@ -566,6 +571,19 @@ mod tests {
         do_ok_test("", &[]);
     }
 
+    #[test]
+    fn test_statement_separators() {
+        do_ok_test(
+            "a=1\nb=2:c=3:' A comment: that follows\nd=4",
+            &[
+                Statement::Assignment(VarRef::new("a", VarType::Auto), Expr::Integer(1)),
+                Statement::Assignment(VarRef::new("b", VarType::Auto), Expr::Integer(2)),
+                Statement::Assignment(VarRef::new("c", VarType::Auto), Expr::Integer(3)),
+                Statement::Assignment(VarRef::new("d", VarType::Auto), Expr::Integer(4)),
+            ],
+        );
+    }
+
     #[test]
     fn test_assignments() {
         do_ok_test(
@@ -707,6 +725,10 @@ mod tests {
             "1 / 2",
             Divide(Box::from(Integer(1)), Box::from(Integer(2))),
         );
+        do_expr_ok_test(
+            "1 MOD 2",
+            Modulo(Box::from(Integer(1)), Box::from(Integer(2))),
+        );
     }
 
     #[test]
